#pragma once

#include "classicnote.h"
#include "initializers/arrownoteinitializer.h"

class ClassicArrowNote : public ClassicNote
{
public:
    explicit ClassicArrowNote(ArrowNoteInitializer&& init);
    virtual ~ClassicArrowNote() = default;

    virtual void putToGame() override;
    virtual void update(const microsec &music_offset) override;
    virtual void input(PlayerInput&& inputdata) override;

    virtual void draw(const ClassicGameGraphicsManager * const manager, sf::RenderTarget& target, sf::RenderStates states) const override;
    virtual void setGraphics(ClassicGameGraphicsManager * const manager, TimeRange&& range) override;

    bool allElementsPressed() const;
    bool isPressedAs(sf::Keyboard::Key key) const;
    inline bool isHold() const;

    struct ArrowElement
    {
        std::shared_ptr<ClassicSprite> sprite;
        std::array<std::shared_ptr<ClassicAnimationScenario>, 5> animations;
        sf::Keyboard::Key pressed_as = sf::Keyboard::Unknown;

        Coordinates coordinates;
        std::vector<Coordinates> falling_curve_interpolation;
        std::array<sf::Keyboard::Key, 2> keys;
        Type type = Type::NONE;
        bool pressed = false;

        // Each note may consist of several buttons.
        // For example, ↑ → or ↓ → ←
        // Note Element represents this idea.
    };

private:
    std::vector<ArrowElement> _elements;
    bool _is_hold;
};

using ArrowElements = std::vector<ClassicArrowNote::ArrowElement>;
